// PaintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Graphics.h"
#include "PaintDlg.h"
#include "GraphicsDlg.h" //   
#include "math.h"
// CPaintDlg dialog

IMPLEMENT_DYNAMIC(CPaintDlg, CDialog)

const COLORREF CPaintDlg::m_crColors[8] = { 

RGB( 0, 0, 0), // Black 
RGB( 0, 0, 255), // Blue 
RGB( 0, 255, 0), // Green 
RGB( 0, 255, 255), // Cyan 
RGB( 255, 0, 0), // Red 
RGB( 255, 0, 255), // Magenta 
RGB( 255, 255, 0), // Yellow 
RGB( 255, 255, 255) // White 
}; 

CPaintDlg::CPaintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPaintDlg::IDD, pParent)
{
}

CPaintDlg::~CPaintDlg()
{
}

void CPaintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPaintDlg, CDialog)
	ON_WM_PAINT()
END_MESSAGE_MAP()

//////////////////////////////
/////////////////////////////
void CPaintDlg::Sierpinsky(CPaintDC *p, int order, int x, int y, int len)
{
  const double s3d2 = sqrt(3)/2;
 CBrush *pOldBrush = NULL;  

  CBrush brSolidBrush(m_crColors[0]);
  pOldBrush = p->SelectObject(&brSolidBrush);

   CPoint ptTriangle[3]; 

   ptTriangle[0].x = x+len/2;
   ptTriangle[0].y = y-len*s3d2;

   ptTriangle[1].x = x+len;
   ptTriangle[1].y = y;

   ptTriangle[2].x = x;
   ptTriangle[2].y = y;

  p->Polygon(ptTriangle,3);

  if(order >0)
  {
  CBrush brSolidBrush(m_crColors[7]);
  p->SelectObject(&brSolidBrush);

  CPoint ptTriangle1[3]; 

   ptTriangle1[0].x = x+len/4;
   ptTriangle1[0].y = y-len*s3d2/2;

   ptTriangle1[1].x = x+3*len/4;
   ptTriangle1[1].y = y-len*s3d2/2;

   ptTriangle1[2].x = x+len/2;
   ptTriangle1[2].y = y;

  p->Polygon(ptTriangle1,3);
  p->SelectObject(pOldBrush);

  Sierpinsky(p, order - 1,x,y,len/2);
  Sierpinsky(p, order - 1,x+len/2,y,len/2);
  Sierpinsky(p, order - 1,x+len/4,y-len*s3d2/2,len/2);

  }//end if
}
////////////////////////////
///////////////////////////
void CPaintDlg::Dragon(CPaintDC* p, int order)
{
	CObArray* line = new CObArray; 
	line = GetDragonPolyline(p, order);

	CPoint* pt = new CPoint(0,0); 
	pt = (CPoint*)line->GetAt(0);
	p->MoveTo(pt->x, pt->y);

	for(unsigned i =1; i < (line->GetCount()); i++){
		CPoint* pt1 = new CPoint(0,0);
		pt1	= (CPoint*)line->GetAt(i);
		p->LineTo(pt1->x, pt1->y);
	}
}

////////////////////////////
///////////////////////////
void CPaintDlg::DrawLine(CPaintDC* pdc, int iColor, int num)
{
    // Declare and create the pens
    CPen pnSolidPen (PS_SOLID, 1, m_crColors[iColor]);
    
    // Get the drawing area
    CRect rRect;
    GetClientRect(rRect);
    rRect.NormalizeRect();

    // Calculate the distance between each of the lines
    CPoint ptStart;
    CPoint ptEnd;
    int iDist = rRect.Height() / 8;
    CPen* pOldPen = NULL;
    // Specify the starting points
    ptStart.y = rRect.top;
    ptStart.x = rRect.left;
    ptEnd.y = ptStart.y;
    ptEnd.x = rRect.right;

        pOldPen = pdc->SelectObject(&pnSolidPen);

        // Move down to the next position
        ptStart.y = ptStart.y + 8*iDist;
        ptEnd.y = ptStart.y;

		switch(num){
			case 0: Koch(pdc, 4, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
				break;
			case 1: Sierpinsky(pdc, 4, ptStart.x, ptStart.y, ((rRect.right-rRect.left)/2));	
				break;
			case 2: Dragon(pdc, 12);
				break;
			default : break;
		}
    // Select the original pen
    pdc->SelectObject(pOldPen);
}

void CPaintDlg::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	// Get a pointer to the parent window
    CGraphicsDlg *pWnd = (CGraphicsDlg*)GetParent();
    // Do we have a valid pointer?
       //     ?
    if (pWnd)
    {
            // What we drawing?
			if (pWnd->m_iShape == 0)  
				DrawLine(&dc, pWnd->m_iColor, 0);
			if (pWnd->m_iShape == 1) 
				DrawLine(&dc, pWnd->m_iColor, 1);
			if (pWnd->m_iShape == 2) 
                DrawLine(&dc, pWnd->m_iColor, 2);
    }
    // Do not call CDialog::OnPaint() for painting messages
}
